;  ------------------------------------------------------------------

;  Procedures which demonstrate sequential numbering techniques

;  ------------------------------------------------------------------

Proc Main_Menu()

Private choice_1

choice_1 = "Basic"
While True
   Clear
   Reset
   test_name = ""
   ShowMenu
      "Basic":       "Basic SCAN loops to fill in an unkeyed table",
      "Renumber":    "SCAN loops to renumber records",
      "Wait":        "Record numbering in WAIT RECORD programs",
      "MultiTable":  "Record numbering in a multi-table form",
      "SetKeys":     "Use SETKEY to establish record-numbering hot-keys",
      "Original":    "Restore the tables to their original condition",
      "Quit":        "All done"
      default choice_1
      to choice_1

   Clear
   Switch
      Case choice_1="Basic":        Basic_Menu()
      Case choice_1="Renumber":     Renumber_Menu()
      Case choice_1="Wait":         Wait_Menu()
      Case choice_1="MultiTable":   MultiTable_Menu()
      Case choice_1="SetKeys":      SetKeys()
      Case choice_1="Original":     Restore_Tables()
      Otherwise:                    QuitLoop
   EndSwitch
EndWhile

EndProc

;  ------------------------------------------------------------------

Proc Basic_Menu()

Private choice_2

choice_2 = "Basic"
While True
   Clear
   Reset
   test_name = ""
   ShowMenu
      "Basic":       "Scan loop with [ID #]=[#]",
      "Recno":       "Scan loop with [ID #]=Recno()",
      "CoEdit":      "Scan loop with Edit instead of CoEdit",
      "MoveTo":      "Scan loop with [] instead of [ID #]",
      "Alpha":       "Scan loop filling in an alpha field"
      default choice_2
      to choice_2

   Clear
   Switch
      Case choice_2="Basic":     Basic_Scan()
      Case choice_2="Recno":     Basic_Recno()
      Case choice_2="CoEdit":    Basic_CoEdit()
      Case choice_2="MoveTo":    Basic_MoveTo()
      Case choice_2="Alpha":     Basic_Alpha()
      Otherwise:                 QuitLoop
   EndSwitch
EndWhile

EndProc

;  ------------------------------------------------------------------

Proc Renumber_Menu()

Private choice_2

choice_2 = "Partial"
While True
   Clear
   Reset
   test_name = ""
   ShowMenu
      "Partial":     "Fill in blank ID #'s",
      "OverWrite":   "Change existing ID #'s, in Edit mode",
      "CoEditOver":  "Change existing ID #'s, in CoEdit mode"
      default choice_2
      to choice_2

   Clear
   Switch
      Case choice_2="Partial":      Renumber_Partial()
      Case choice_2="OverWrite":    Renumber_OverWrite(False)
      Case choice_2="CoEditOver":   Renumber_OverWrite(True)
      Otherwise:                    QuitLoop
   EndSwitch
EndWhile

EndProc

;  ------------------------------------------------------------------

Proc Wait_Menu()

Private choice_2

choice_2 = "Single"
While True
   Clear
   Reset
   test_name = ""
   ShowMenu
      "Single":      "Method that works in a single-user system",
      "RetryMulti":  "In a multi-user system, retry number failures",
      "Counter":     "Store the highest number in a separate COUNTER table",
      "PutBack":     "When a record is deleted, put its # back in COUNTER",
      "List":        "Store available numbers in a separate list table"
      default choice_2
      to choice_2

   Clear
   Switch
      Case choice_2="Single":       Wait_Single()
      Case choice_2="RetryMulti":   Wait_Retry()
      Case choice_2="Counter":      Wait_Counter()
      Case choice_2="PutBack":      Wait_Put_Back()
      Case choice_2="List":         Wait_List()
      Otherwise:                    QuitLoop
   EndSwitch
EndWhile

EndProc

;  ------------------------------------------------------------------

Proc Basic_Scan()

;  The basic scan loop for filling in sequential numbers

test_name = "Basic_Scan:  [ID #]=[#]"
Edit "Names1"
Before_Results()
Scan
   [ID #] = [#]
EndScan
After_Results()
Do_it!
Wait_After()

EndProc

;  ------------------------------------------------------------------

Proc Basic_Recno()

;  The basic scan loop for filling in sequential numbers,
;  using Recno() instead of [#]

test_name = "Basic_Recno:  [ID #]=Recno()"
Edit "Names1"
Before_Results()
Scan
   [ID #] = Recno()
EndScan
After_Results()
Do_it!
Wait_After()

EndProc

;  ------------------------------------------------------------------

Proc Basic_CoEdit()

;  The basic scan loop for filling in sequential numbers,
;  using CoEdit instead of Edit

test_name = "Basic_CoEdit:  CoEdit / [ID #]=[#]"
CoEdit "Names1"
Before_Results()
Scan
   [ID #] = [#]
EndScan
After_Results()
Do_it!
Wait_After()

EndProc

;  ------------------------------------------------------------------

Proc Basic_MoveTo()

;  The basic scan loop for filling in sequential numbers,
;  using MoveTo and [] instead of [ID #]

test_name = "Basic_MoveTo:  MoveTo [ID #] / []=[#]"
CoEdit "Names1"
MoveTo [ID #]
Before_Results()
Scan
   [] = [#]
EndScan
After_Results()
Do_it!
Wait_After()

EndProc

;  ------------------------------------------------------------------

Proc Basic_Alpha()

;  The basic scan loop for filling in sequential numbers
;  into an alphanumeric field

test_name = "Basic_Alpha:  [ID #] is an alpha field"
Edit "Names3"
Before_Results()
Scan
   [ID #] = substr(format("w6,ez",[#]),2,5)
EndScan
After_Results()
Do_it!
Wait_After()

EndProc

;  ------------------------------------------------------------------

Proc Renumber_Partial()

;  The scan loop for filling in sequential numbers only in blank records

test_name = "Renumber_Partial:  Fill in only if ID # is blank"
Restore_Tables()
CoEdit "Names1"
Scan
   If rand()>0.5 then
      [ID #] = [#]
   Endif
EndScan
Do_it!
h = cmax("Names1","ID #")
If h="Error" then h=0 Endif

Edit "Names1"
Before_Results()
Wait_Before()
Scan for isblank([ID #])
   h = h+1
   [ID #] = h
EndScan
After_Results()
Do_it!
Wait_After()

EndProc

;  ------------------------------------------------------------------

Proc Renumber_OverWrite(is_coedit)

;  A scan loop that runs into trouble when re-numbering keyed records.

test_name = "Renumber_Overwrite:  Dangerous re-numbering of keyed table"
If is_coedit then
   test_name = test_name + " - COEDIT"
Else
   test_name = test_name + " - EDIT"
Endif
Restore_Tables()
CoEdit "Names2"
Scan
   Switch
      Case [#]<3:    [ID #] = [#]
      Case [#]=3:    [ID #] = [#] - 0.5
      Otherwise:     [ID #] = [#] - 1.0
   EndSwitch
EndScan
Do_it!

If is_coedit then
   CoEdit "Names2"
Else
   Edit "Names2"
Endif
Before_Results()
Wait_Before()
Scan
   [ID #] = [#]
EndScan
Do_it!
After_Results()
Wait_After()

EndProc

;  ------------------------------------------------------------------

Proc Wait_Single()

;  Filling in sequential key values in a single-user WAIT RECORD program

test_name = "Wait_Single:  Single-user WAIT RECORD"
CoEdit "Names2"
End
h = [ID #]
If isblank(h) then h=0 Endif

While True
   ImageRights Update
   Wait Record
      Prompt "[Ins]=Enter a new record",
             "[F2]=Done"
      Until "F2", "Ins", "Del", "Home", "End", 
            "Up", "Down", "PgUp", "PgDn",
            "LockKey"
   wait_key = retval
   ImageRights
   If wait_key="LockKey" then Beep Loop Endif
   If isblank([ID #]) and 
      recordstatus("New") and 
      recordstatus("Modified") and
      wait_key<>"Del" then
         h = h+1
         [ID #] = h
   Endif
   KeyPress wait_key
   If wait_key="F2" then QuitLoop Endif
EndWhile
Do_it!

EndProc

;  ------------------------------------------------------------------

Proc Wait_Retry()

;  Filling in sequential key values in a multi-user WAIT RECORD program;
;  If the number is already in use, the program keeps incrementing until
;  it finds an available number.

test_name = "Wait_Retry:  Multi-user WAIT RECORD with retry"
CoEdit "Names2"
End
h = [ID #]
If isblank(h) then h=0 Endif
wait_msg = ""

While True
   ImageRights Update
   Wait Record
      Prompt "[Ins]=Enter a new record",
             "[F2]=Done"
      Message wait_msg
      Until "F2", "Ins", "Del", "Home", "End", 
            "Up", "Down", "PgUp", "PgDn",
            "LockKey", "KeyLookup"
   wait_key = retval
   wait_msg = ""
   ImageRights
   If wait_key="LockKey" or wait_key="KeyLookup" then 
      Beep Loop 
   Endif
   If isblank([ID #]) and 
      recordstatus("New") and 
      recordstatus("Modified") and
      wait_key<>"Del" then
         While True
            h = h+1
            [ID #] = h
            LockRecord
            If retval then QuitLoop Endif
            If not recordstatus("KeyViol") then
               wait_msg = errormessage()
               QuitLoop
            Endif
         EndWhile
         If not isblank(wait_msg) then Loop Endif
   Endif
   KeyPress wait_key
   If wait_key="F2" then QuitLoop Endif
EndWhile
Do_it!

EndProc

;  ------------------------------------------------------------------

Proc Wait_Counter()

;  Filling in sequential key values in a multi-user WAIT RECORD program;
;  Using a COUNTER table to retrieve the next available number.

test_name = "Wait_Counter:  Multi-user WAIT RECORD with counter"
View "Counter"
CoEdit "Names2"

While True
   ImageRights Update
   Wait Record
      Prompt "[Ins]=Enter a new record",
             "[F2]=Done"
      Until "F2", "Ins", "Del", "Home", "End", 
            "Up", "Down", "PgUp", "PgDn",
            "LockKey"
   wait_key = retval
   ImageRights
   If wait_key="LockKey" then Beep Loop Endif
   If isblank([ID #]) and 
      recordstatus("New") and 
      recordstatus("Modified") and
      wait_key<>"Del" then
         MoveTo "Counter"
         LockRecord
         While not retval LockRecord EndWhile
         h = [Highest ID #]+1
         [Highest ID #] = h
         UnlockRecord
         MoveTo "Names2"
         [ID #] = h
   Endif
   KeyPress wait_key
   If wait_key="F2" then QuitLoop Endif
EndWhile
Do_it!

EndProc

;  ------------------------------------------------------------------

Proc Wait_Put_Back()

;  Filling in sequential key values in a multi-user WAIT RECORD program;
;  Using a COUNTER table to retrieve the next available number;
;  When a record is deleted, put its number back into the COUNTER table
;  so that it can be re-used.

test_name = "Wait_Put_Back:  Multi-user WAIT RECORD with counter & put-back"
View "Counter"
CoEdit "Names2"

While True
   ImageRights Update
   Wait Record
      Prompt "[Ins]=Enter a new record",
             "[F2]=Done"
      Until "F2", "Ins", "Del", "Home", "End", 
            "Up", "Down", "PgUp", "PgDn",
            "LockKey"
   wait_key = retval
   ImageRights
   If wait_key="LockKey" then Beep Loop Endif
   If isblank([ID #]) and 
      recordstatus("New") and 
      recordstatus("Modified") and
      wait_key<>"Del" then
         MoveTo "Counter"
         LockRecord
         While not retval LockRecord EndWhile
         h = [Highest ID #]+1
         [Highest ID #] = h
         UnlockRecord
         MoveTo "Names2"
         [ID #] = h
   Endif
   If wait_key="Del" and not isblank([ID #]) then
      h = [ID #]
      MoveTo "Counter"
      LockRecord
      While not retval LockRecord EndWhile
      If [Highest ID #]=h+1 then
         [Highest ID #] = h
      Endif
      UnlockRecord
      MoveTo "Names2"
   Endif   
   KeyPress wait_key
   If wait_key="F2" then QuitLoop Endif
EndWhile
Do_it!

EndProc

;  ------------------------------------------------------------------

Proc Wait_List()

;  Filling in sequential key values in a multi-user WAIT RECORD program;
;  Using a SOURCE table, which is a list of available numbers,
;  to retrieve the next available number;
;  When a record is deleted, put its number back into the SOURCE table
;  so that it can be re-used.

test_name = "Wait_List:  Multi-user WAIT RECORD with numbers in a list table"
View "Source"
CoEdit "Names2"

While True
   ImageRights Update
   Wait Record
      Prompt "[Ins]=Enter a new record",
             "[F2]=Done"
      Until "F2", "Ins", "Del", "Home", "End", 
            "Up", "Down", "PgUp", "PgDn",
            "LockKey"
   wait_key = retval
   ImageRights
   If wait_key="LockKey" then Beep Loop Endif
   If isblank([ID #]) and 
      recordstatus("New") and 
      recordstatus("Modified") and
      wait_key<>"Del" then
         MoveTo "Source"
         Home
         LockRecord
         While not retval LockRecord EndWhile
         h = [ID #]
         Del
         UnlockRecord
         MoveTo "Names2"
         [ID #] = h
   Endif
   If wait_key="Del" and not isblank([ID #]) then
      h = [ID #]
      MoveTo "Source"
      Ins
      [ID #] = h
      UnlockRecord
      MoveTo "Names2"
   Endif   
   KeyPress wait_key
   If wait_key="F2" then QuitLoop Endif
EndWhile
Do_it!

EndProc

;  ------------------------------------------------------------------

Proc SetKeys()

;  Use the SETKEY command to establish record-numbering hot-keys.

Private k

;  Initialize the highest number

SetKey "F22" End h=[ID #] If isblank(h) then h=0 Endif

;  Fill in the next highest number (single-user version)

SetKey "F21" If isblank([ID #]) then h=h+1 [ID #]=h Endif

;  Fill in the next highest number (multi-user version)

SetKey "F23" If isblank([ID #]) then retval=False While not retval h=h+1 [ID #]=h LockRecord EndWhile Endif

;  Display the hot keys

Clear
@ 3,0
Text

   [Ctrl-F2] =    Initialize "H" to the highest [ID #] in use
   [Ctrl-F1] =    Fill in [ID #] with next available value (single-user)
   [Ctrl-F3] =    Fill in [ID #] with next available value (multi-user)

EndText
Message "Press any key to continue..."
k = getchar()

EndProc

;  ------------------------------------------------------------------

Proc Before_Results()

;  Capture information about the environment before the test

before_mode = sysmode()
before_nr   = nrecords(table())
before_r    = recno()
before_fd   = field()
before_id   = [ID #]

EndProc

;  ------------------------------------------------------------------

Proc After_Results()

;  Capture information about the environment after the test

after_mode = sysmode()
after_nr   = nrecords(table())
after_r    = recno()
after_fd   = field()
after_id   = [ID #]

EndProc

;  ------------------------------------------------------------------

Proc Display_Results()

;  Display the test results

Private k

Clear

@ 1,0  ?? test_name

@ 3,15 ?? "Before"
@ 3,25 ?? "After"
@ 4,15 ?? "--------"
@ 4,25 ?? "--------"

@ 05,0  ?? "SysMode"
@ 05,15 ?? before_mode
@ 05,25 ?? after_mode
@ 06,0  ?? "# Records"
@ 06,15 ?? before_nr
@ 06,25 ?? after_nr
@ 07,0  ?? "Record #"
@ 07,15 ?? before_r
@ 07,25 ?? after_r
@ 08,0  ?? "Field"
@ 08,15 ?? before_fd
@ 08,25 ?? after_fd
@ 09,0  ?? "ID # Value"
@ 09,15 ?? before_id
@ 09,25 ?? after_id

Message "Press any key to continue"
k = getchar()
Clear

EndProc

;  ------------------------------------------------------------------

Proc Wait_Before()

;  Display the table before a procedure

Wait Table
   Prompt "This is the table before the test is performed",
         "[F2]=Proceed"
   Until "F2"

EndProc

;  ------------------------------------------------------------------

Proc Wait_After()

;  Display the results of a procedure

While True
   Wait Table
      Prompt "[F1]=Review before-and-after parameters",
            "[F2]=Return to menu,  [R]=Restore original tables, "+
            "[Esc]=Return to Paradox"
      Until "F1", "F2", "R", "r", "Esc"

   retval = upper(retval)
   Switch
      Case retval="F1":    Display_Results()
      Case retval="F2":    QuitLoop
      Case retval="R":     Restore_Tables()
                           QuitLoop
      Case retval="ESC":   Quit
   EndSwitch
EndWhile

Release Vars test_name

EndProc

;  ------------------------------------------------------------------

Proc Restore_Tables()

;  Restore all tables to their original condition

Copy "Orig1" "Names1"
Copy "Orig2" "Names2"
Copy "Orig3" "Names3"
Copy "OrigC" "Counter"
Copy "OrigS" "Source"

EndProc

;  ------------------------------------------------------------------

SetKey "F30" Play "SeqNum"
Main_Menu()

;  ------------------------------------------------------------------
